@@ -43,21 +43,22 @@ class Backend:
         self.settingsFile = os.path.join(prefsDir, "user-settings.conf")
         self.appPrefsDir = os.path.join(prefsDir, "app-settings")
 
+        self._defaultProfiles = {'default': { 'profile':  settings.profile,
+                                                          'pronunciations': {},
+                                                          'keybindings': {}
+                                            }
+                                }
+
     def saveDefaultSettings(self, general, pronunciations, keybindings):
         """ Save default settings for all the properties from
             orca.settings. """
-        defaultProfiles = {'default': { 'profile':  settings.profile,
-                                                    'pronunciations': {},
-                                                    'keybindings': {}
-                                      }
-                          }
         prefs = {'general': general,
-                 'profiles': defaultProfiles,
+                 'profiles': self._defaultProfiles,
                  'pronunciations': pronunciations,
                  'keybindings': keybindings}
 
         self.general = general
-        self.profiles = defaultProfiles
+        self.profiles = self._defaultProfiles
         self.pronunciations = pronunciations
         self.keybindings = keybindings
 
@@ -65,8 +66,8 @@ class Backend:
         dump(prefs, settingsFile, indent=4)
         settingsFile.close()
 
-    def getAppSettings(self, appName):
-        fileName = os.path.join(self.appPrefsDir, "%s.conf" % appName)
+    def get_app_settings(self, appName):
+        fileName = os.path.join(self.appPrefsDir, f"{appName}.conf")
         if os.path.exists(fileName):
             settingsFile = open(fileName, 'r')
             prefs = load(settingsFile)
@@ -77,14 +78,14 @@ class Backend:
         return prefs
 
     def saveAppSettings(self, appName, profile, general, pronunciations, keybindings):
-        prefs = self.getAppSettings(appName)
+        prefs = self.get_app_settings(appName)
         profiles = prefs.get('profiles', {})
         profiles[profile] = {'general': general,
                              'pronunciations': pronunciations,
                              'keybindings': keybindings}
         prefs['profiles'] = profiles
 
-        fileName = os.path.join(self.appPrefsDir, "%s.conf" % appName)
+        fileName = os.path.join(self.appPrefsDir, f"{appName}.conf")
         settingsFile = open(fileName, 'w')
         dump(prefs, settingsFile, indent=4)
         settingsFile.close()
@@ -118,11 +119,15 @@ class Backend:
         self.keybindings = prefs['keybindings']
         self.profiles = prefs['profiles'].copy()
 
-    def getGeneral(self, profile='default'):
+    def getGeneral(self, profile=None):
         """ Get general settings from default settings and
             override with profile values. """
         self._getSettings()
         generalSettings = self.general.copy()
+        defaultProfile = generalSettings.get('startingProfile',
+                                             ['Default', 'default'])
+        if profile is None:
+            profile = defaultProfile[1]
         profileSettings = self.profiles[profile].copy()
         for key, value in profileSettings.items():
             if key == 'voices':
@@ -133,10 +138,10 @@ class Backend:
         try:
             generalSettings['activeProfile'] = profileSettings['profile']
         except KeyError:
-            generalSettings['activeProfile'] = ["Default", "default"] 
+            generalSettings['activeProfile'] = defaultProfile
         return generalSettings
 
-    def getPronunciations(self, profile='default'):
+    def get_pronunciations(self, profile='default'):
         """ Get pronunciation settings from default settings and
             override with profile values. """
         self._getSettings()
@@ -146,7 +151,7 @@ class Backend:
             pronunciations = profileSettings['pronunciations']
         return pronunciations
 
-    def getKeybindings(self, profile='default'):
+    def get_keybindings(self, profile='default'):
         """ Get keybindings settings from default settings and
             override with profile values. """
         self._getSettings()
@@ -156,7 +161,7 @@ class Backend:
             keybindings = profileSettings['keybindings']
         return keybindings
 
-    def isFirstStart(self):
+    def is_first_start(self):
         """ Check if we're in first start. """
  
         return not os.path.exists(self.settingsFile)
@@ -171,14 +176,7 @@ class Backend:
             settingsFile.truncate()
             dump(prefs, settingsFile, indent=4)
 
-    def setFirstStart(self, value=False):
-        """Set firstStart. This user-configurable settting is primarily
-        intended to serve as an indication as to whether or not initial
-        configuration is needed."""
-        self.general['firstStart'] = value
-        self._setProfileKey('firstStart', value)
-
-    def availableProfiles(self):
+    def available_profiles(self):
         """ List available profiles. """
         self._getSettings()
         profiles = []
@@ -188,3 +186,23 @@ class Backend:
             profiles.append(profileDict.get('profile'))
 
         return profiles
+
+    def remove_profile(self, profile):
+        """Remove an existing profile"""
+        def removeProfileFrom(dict):
+            del dict[profile]
+            # if we removed the last profile, restore the default ones
+            if len(dict) == 0:
+                for profileName in self._defaultProfiles:
+                    dict[profileName] = self._defaultProfiles[profileName].copy()
+
+        if profile in self.profiles:
+            removeProfileFrom(self.profiles)
+
+        with open(self.settingsFile, 'r+') as settingsFile:
+            prefs = load(settingsFile)
+            if profile in prefs['profiles']:
+                removeProfileFrom(prefs['profiles'])
+                settingsFile.seek(0)
+                settingsFile.truncate()
+                dump(prefs, settingsFile, indent=4)
