@@ -19,6 +19,8 @@
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
 
+# pylint: disable=duplicate-code
+
 """Custom chat module for Smuxi."""
 
 __id__        = "$Id$"
@@ -28,21 +30,19 @@ __copyright__ = "Copyright (c) 2018 Igalia, S.L."
 __license__   = "LGPL"
 
 
-import orca.chat as chat
+from orca import chat
 from orca.ax_object import AXObject
 from orca.ax_utilities import AXUtilities
 
 
 class Chat(chat.Chat):
-
-    def __init__(self, script, buddyListAncestries):
-        super().__init__(script, buddyListAncestries)
+    """Custom chat module for Smuxi."""
 
     def isFocusedChat(self, obj):
         """Returns True if we plan to treat this chat as focused."""
 
-        pageTab = AXObject.find_ancestor(obj, AXUtilities.is_page_tab)
-        if pageTab is None:
+        page_tab = AXObject.find_ancestor(obj, AXUtilities.is_page_tab)
+        if page_tab is None:
             return super().isFocusedChat(obj)
 
-        return AXUtilities.is_showing(pageTab)
+        return AXUtilities.is_showing(page_tab)
